@@ -1,4 +1,4 @@
-# $Id: HTTP.pm 310 2007-02-27 07:15:01Z rcaputo $
+# $Id: HTTP.pm 361 2009-02-19 07:14:19Z rcaputo $
 
 package POE::Component::Client::HTTP;
 
@@ -11,7 +11,7 @@ use constant DEBUG      => 0;
 use constant DEBUG_DATA => 0;
 
 use vars qw($VERSION);
-$VERSION = '0.82';
+$VERSION = '0.88';
 
 use Carp qw(croak);
 use HTTP::Response;
@@ -84,11 +84,6 @@ while (my ($encoding, $filter) = each %te_possible_filters) {
 #  qw(x-bzip2 gzip x-gzip deflate compress chunked identity)
 #);
 
-# Set default accept encoding here, but DON'T SET later, if the
-# request is streaming.
-
-my $accept_encoding = Net::HTTP::Methods::zlib_ok() ? 'gzip' : '';
-
 my %supported_schemes = (
   http  => 1,
   https => 1,
@@ -113,6 +108,7 @@ sub spawn {
   my $alias = delete $params{Alias};
   $alias = 'weeble' unless defined $alias and length $alias;
 
+  my $bind_addr = delete $params{BindAddr};
   my $cm = delete $params{ConnectionManager};
 
   my $request_factory = POE::Component::Client::HTTP::RequestFactory->new(
@@ -153,6 +149,7 @@ sub spawn {
       factory      => $request_factory,
       cm           => $cm,
       is_shut_down => 0,
+      bind_addr    => $bind_addr,
     },
   );
 
@@ -171,6 +168,7 @@ sub _poco_weeble_start {
   # have to do this here because it wants a current_session
   $heap->{cm} = POE::Component::Client::Keepalive->new(
     timeout => $heap->{factory}->timeout,
+    $heap->{bind_addr} ? (bind_address => $heap->{bind_addr}) : (),
   ) unless ($heap->{cm});
 }
 
@@ -184,6 +182,7 @@ sub _poco_weeble_stop {
 
   foreach my $request_rec (values %$request) {
     $request_rec->remove_timeout();
+    delete $heap->{ext_request_to_int_id}->{$request_rec->[REQ_HTTP_REQUEST]};
   }
 
   DEBUG and warn "Client::HTTP (alias=$heap->{alias}) stopped.";
@@ -225,9 +224,13 @@ sub _poco_weeble_request {
        . "Unsupported URI scheme\n"
        . "</BODY>\n"
        . "</HTML>\n"
-      );
+    );
     $rsp->request($http_request);
-    $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
+    if (ref $response_event) {
+      $response_event->postback->($rsp);
+    } else {
+      $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
+    }
     return;
   }
 
@@ -244,7 +247,11 @@ sub _poco_weeble_request {
        . "</HTML>\n"
       );
     $rsp->request($http_request);
-    $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
+    if (ref $response_event) {
+      $response_event->postback->($rsp);
+    } else {
+      $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
+    }
     return;
   }
 
@@ -252,23 +259,12 @@ sub _poco_weeble_request {
     POE::Component::Client::HTTP::RequestFactory->parse_proxy($proxy_override);
   }
 
-  # Add an Accept-Encoding header if streaming is not enabled and the
-  # header doesn't already exist.
-  if (
-    !$heap->{factory}->is_streaming and
-    !$heap->{factory}->max_response_size and # or for limited chunk sizes
-    !defined($http_request->header('Accept-Encoding')) and
-    length($accept_encoding)
-  ) {
-    $http_request->header('Accept-Encoding', $accept_encoding);
-  }
-
   my $request = $heap->{factory}->create_request(
     $http_request, $response_event, $tag, $progress_event,
     $proxy_override, $sender
   );
   $heap->{request}->{$request->ID} = $request;
-  $heap->{request_to_id}->{$http_request} = $request->ID;
+  $heap->{ext_request_to_int_id}->{$http_request} = $request->ID;
 
   my @timeout;
   if ($heap->{factory}->timeout()) {
@@ -277,22 +273,23 @@ sub _poco_weeble_request {
     );
   }
 
-  my $cm_req_id;
   eval {
-      # get a connection from Client::Keepalive
-      $request->[REQ_CONN_ID] = $heap->{cm}->allocate(
-        scheme  => $request->scheme,
-        addr    => $request->host,
-        port    => $request->port,
-        context => $request->ID,
-        event   => 'got_connect_done',
-        @timeout,
-      );
+    # get a connection from Client::Keepalive
+    $request->[REQ_CONN_ID] = $heap->{cm}->allocate(
+      scheme  => $request->scheme,
+      addr    => $request->host,
+      port    => $request->port,
+      context => $request->ID,
+      event   => 'got_connect_done',
+      @timeout,
+    );
   };
   if ($@) {
-      delete $heap->{request}->{$request->ID};
-      # we can reach here for things like host being invalid.
-      $request->error(400, $@);
+    delete $heap->{request}->{$request->ID};
+    delete $heap->{ext_request_to_int_id}->{$http_request};
+
+    # we can reach here for things like host being invalid.
+    $request->error(400, $@);
   }
 }
 
@@ -307,10 +304,24 @@ sub _poco_weeble_connect_done {
   my $connection = $response->{'connection'};
   my $request_id = $response->{'context'};
 
+  # Can't handle connections if we're shut down.
+  # TODO - How do we still get these?  Were they previously queued or
+  # something?
+  if ($heap->{is_shut_down}) {
+    _internal_cancel(
+      $heap, $request_id, 408, "Request timed out (request canceled)"
+    );
+    return;
+  }
+
   if (defined $connection) {
     DEBUG and warn "CON: request $request_id connected ok...";
 
     my $request = $heap->{request}->{$request_id};
+    unless (defined $request) {
+      DEBUG and warn "CON: ignoring connection for canceled request";
+      return;
+    }
 
     my $block_size = $heap->{factory}->block_size;
 
@@ -352,6 +363,7 @@ sub _poco_weeble_connect_done {
     DEBUG and warn "I/O: removing request $request_id";
     my $request = delete $heap->{request}->{$request_id};
     $request->remove_timeout();
+    delete $heap->{ext_request_to_int_id}->{$request->[REQ_HTTP_REQUEST]};
 
     # Post an error response back to the requesting session.
     $request->connect_error("$operation error $errnum: $errstr");
@@ -381,6 +393,7 @@ sub _poco_weeble_timeout {
 
   DEBUG and warn "T/O: request $request_id has timer ", $request->timer;
   $request->remove_timeout();
+  delete $heap->{ext_request_to_int_id}->{$request->[REQ_HTTP_REQUEST]};
 
   # There's a wheel attached to the request.  Shut it down.
   if (defined(my $wheel = $request->wheel())) {
@@ -390,9 +403,9 @@ sub _poco_weeble_timeout {
     # Shut down the connection so it's not reused.
     $wheel->shutdown_input();
     delete $heap->{wheel_to_request}->{$wheel_id};
-    delete $heap->{request_to_id}->{$request->[REQ_REQUEST]};
   }
 
+
   DEBUG and do {
     die( "T/O: request $request_id is unexpectedly zero" )
       unless $request->[REQ_STATE];
@@ -435,10 +448,10 @@ sub _poco_weeble_io_flushed {
   );
 
   my $request = $heap->{request}->{$request_id};
-  
+
   # Read content to send from a callback
-  if ( ref $request->[REQ_REQUEST]->content() eq 'CODE' ) {
-    my $callback = $request->[REQ_REQUEST]->content();
+  if ( ref $request->[REQ_HTTP_REQUEST]->content() eq 'CODE' ) {
+    my $callback = $request->[REQ_HTTP_REQUEST]->content();
 
     my $buf = eval { $callback->() };
 
@@ -454,7 +467,7 @@ sub _poco_weeble_io_flushed {
       return;
     }
   }
-  
+
   $request->[REQ_STATE] ^= RS_SENDING;
   $request->[REQ_STATE] = RS_IN_HEAD;
   # XXX - Removed a second time.  The first time was in version 0.53,
@@ -471,81 +484,94 @@ sub _poco_weeble_io_error {
   my ($kernel, $heap, $operation, $errnum, $errstr, $wheel_id) =
     @_[KERNEL, HEAP, ARG0..ARG3];
 
-  DEBUG and
-    warn "I/O: wheel $wheel_id encountered $operation error $errnum: $errstr";
+  DEBUG and warn(
+    "I/O: wheel $wheel_id encountered $operation error $errnum: $errstr"
+  );
 
   # Drop the wheel.
   my $request_id = delete $heap->{wheel_to_request}->{$wheel_id};
   #K or die "!!!: unexpectedly undefined request ID" unless defined $request_id;
 
-  if ($request_id) {
+  # There was no corresponding request?  Nothing left to do here.
+  return unless $request_id;
 
-    DEBUG and warn "I/O: removing request $request_id";
-    my $request = delete $heap->{request}->{$request_id};
-    delete $heap->{request_to_id}{$request->[REQ_REQUEST]};
-    $request->remove_timeout;
+  DEBUG and warn "I/O: removing request $request_id";
+  my $request = delete $heap->{request}->{$request_id};
+  $request->remove_timeout;
+  delete $heap->{ext_request_to_int_id}{$request->[REQ_HTTP_REQUEST]};
 
-    # Otherwise the remote end simply closed.  If we've got a
-    # pending response, then post it back to the client.
-    DEBUG and warn "STATE is ", $request->[REQ_STATE];
+  # Otherwise the remote end simply closed.  If we've got a pending
+  # response, then post it back to the client.
+  DEBUG and warn "STATE is ", $request->[REQ_STATE];
 
-    # except when we're redirected
-    return if ($request->[REQ_STATE] & RS_REDIRECTED);
+  # Except when we're redirected.  In this case, the connection was but
+  # one step towards our destination.
+  return if ($request->[REQ_STATE] & RS_REDIRECTED);
 
-    # If there was a non-zero error, then something bad happened.  Post
-    # an error response back, if we haven't posted anything before.
-    if ($errnum) {
-      unless ($request->[REQ_STATE] & RS_POSTED) {
-        $request->error(400, "$operation error $errnum: $errstr");
-      }
-      return;
+  # If there was a non-zero error, then something bad happened.  Post
+  # an error response back, if we haven't posted anything before.
+  if ($errnum) {
+    unless ($request->[REQ_STATE] & RS_POSTED) {
+      $request->error(400, "$operation error $errnum: $errstr");
     }
-
-    if (
-      $request->[REQ_STATE] & (RS_IN_CONTENT | RS_DONE) and
-      not $request->[REQ_STATE] & RS_POSTED
-    ) {
-      _finish_request($heap, $request, 0);
-      return;
-    }
-    elsif ($request->[REQ_STATE] & RS_POSTED) {
-      DEBUG and warn "I/O: Disconnect, remote keepalive timeout or HTTP/1.0.";
-      return;
-    }
-    elsif (not defined $request->[REQ_RESPONSE]) {
-      # never got a response, check for pending data indicating
-      # a LF-free HTTP 0.9 response
-      my $lines = $request->wheel->get_input_filter()->get_pending();
-      my $text = join '' => @$lines;
-      DEBUG and warn "Got ", length($text), " bytes of data without LF.";
-      if ($text =~ /\S/) {
-        # generate response
-        DEBUG and warn(
-          "Generating HTTP response for HTTP/0.9 response without LF."
-        );
-        $request->[REQ_RESPONSE] = HTTP::Response->new(
-          200, 'OK', [ 'Content-Type' => 'text/html' ], $text
-        );
-        $request->[REQ_RESPONSE]->protocol('HTTP/0.9');
-        $request->[REQ_RESPONSE]->request($request->[REQ_REQUEST]);
-        $request->[REQ_STATE] = RS_DONE;
-        $request->return_response;
-        return;
-      }
-    }
-
-    # We haven't built a proper response.  Send back an error.
-    # Changed to 406 after considering rt.cpan.org 20975.
-    #
-    # 10.4.7 406 Not Acceptable
-    #
-    # The resource identified by the request is only capable of
-    # generating response entities which have content characteristics
-    # not acceptable according to the accept headers sent in the
-    # request.
-
-    $request->error (406, "Response larger than MaxSize - $request_id");
+    return;
   }
+
+  # We seem to have finished with the request.  Send back a response.
+  if (
+    $request->[REQ_STATE] & (RS_IN_CONTENT | RS_DONE) and
+    not $request->[REQ_STATE] & RS_POSTED
+  ) {
+    _finish_request($heap, $request, 0);
+    return;
+  }
+
+  # We have already posted a response, so this is a remote keepalive
+  # timeout or other delayed socket shutdown.  Nothing left to do.
+  if ($request->[REQ_STATE] & RS_POSTED) {
+    DEBUG and warn "I/O: Disconnect, remote keepalive timeout or HTTP/1.0.";
+    return;
+  }
+
+  # We never received a response.
+  if (not defined $request->[REQ_RESPONSE]) {
+    # Check for pending data indicating a LF-free HTTP 0.9 response.
+    my $lines = $request->wheel->get_input_filter()->get_pending();
+    my $text = join '' => @$lines;
+    DEBUG and warn "Got ", length($text), " bytes of data without LF.";
+
+    # If we have data, build and return a response from it.
+    if ($text =~ /\S/) {
+      DEBUG and warn(
+        "Generating HTTP response for HTTP/0.9 response without LF."
+      );
+      $request->[REQ_RESPONSE] = HTTP::Response->new(
+        200, 'OK', [ 'Content-Type' => 'text/html' ], $text
+      );
+      $request->[REQ_RESPONSE]->protocol('HTTP/0.9');
+      $request->[REQ_RESPONSE]->request($request->[REQ_HTTP_REQUEST]);
+      $request->[REQ_STATE] = RS_DONE;
+      $request->return_response;
+      return;
+    }
+
+    # No data received.  This is an incomplete response.
+    $request->error(400, "Incomplete response - $request_id");
+    return;
+  }
+
+  # We haven't built a proper response, and nothing returned by the
+  # server can be turned into a proper response.  Send back an error.
+  # Changed to 406 after considering rt.cpan.org 20975.
+  #
+  # 10.4.7 406 Not Acceptable
+  #
+  # The resource identified by the request is only capable of
+  # generating response entities which have content characteristics
+  # not acceptable according to the accept headers sent in the
+  # request.
+
+  $request->error(406, "Server response is Not Acceptable - $request_id");
 }
 
 # }}} _poco_weeble_io_error
@@ -569,7 +595,7 @@ sub _poco_weeble_io_read {
   my $request = $heap->{request}->{$request_id};
   return unless defined $request;
   DEBUG and warn(
-    "REQUEST $request_id is $request <" . $request->[REQ_REQUEST]->uri . ">"
+    "REQUEST $request_id is $request <" . $request->[REQ_HTTP_REQUEST]->uri . ">"
   );
 
   # Reset the timeout if we get data.
@@ -586,9 +612,9 @@ sub _poco_weeble_io_read {
   # part of the content.
   if ($request->[REQ_STATE] & RS_IN_HEAD) {
     if (defined $input) {
-      $input->request ($request->[REQ_REQUEST]);
+      $input->request ($request->[REQ_HTTP_REQUEST]);
       #warn(
-      #  "INPUT for ", $request->[REQ_REQUEST]->uri, " is \n",$input->as_string
+      #  "INPUT for ", $request->[REQ_HTTP_REQUEST]->uri, " is \n",$input->as_string
       #)
     }
     else {
@@ -605,7 +631,7 @@ sub _poco_weeble_io_read {
     #        Make sure we finish even when it isn't one of these,
     #        but there is no content.
     if (
-      $request->[REQ_REQUEST]->method eq 'HEAD'
+      $request->[REQ_HTTP_REQUEST]->method eq 'HEAD'
       or $input->code =~ /^(?:1|[23]04)/
       or (
         defined($input->content_length())
@@ -617,8 +643,8 @@ sub _poco_weeble_io_read {
         delete $heap->{wheel_to_request}->{$wheel_id};
         if (defined $old_request) {
           DEBUG and warn "I/O: removed request $request_id";
-          delete $heap->{request_to_id}{$old_request->[REQ_REQUEST]};
           $old_request->remove_timeout();
+          delete $heap->{ext_request_to_int_id}{$old_request->[REQ_HTTP_REQUEST]};
           $old_request->[REQ_CONNECTION] = undef;
         }
         return;
@@ -639,7 +665,7 @@ sub _poco_weeble_io_read {
         delete $heap->{wheel_to_request}->{$wheel_id};
         if (defined $old_request) {
           DEBUG and warn "I/O: removed request $request_id";
-          delete $heap->{request_to_id}{$old_request->[REQ_REQUEST]};
+          delete $heap->{ext_request_to_int_id}{$old_request->[REQ_HTTP_REQUEST]};
           $old_request->remove_timeout();
           $old_request->[REQ_CONNECTION]->close();
           $old_request->[REQ_CONNECTION] = undef;
@@ -852,9 +878,9 @@ sub _finish_request {
     if (defined $request) {
       DEBUG and warn "I/O: removing request $request_id";
       $request->remove_timeout();
+      delete $heap->{ext_request_to_int_id}{$request->[REQ_HTTP_REQUEST]};
       if (my $wheel = $request->wheel) {
         delete $heap->{wheel_to_request}->{$wheel->ID};
-        delete $heap->{request_to_id}{$request->[REQ_REQUEST]};
       }
     }
   }
@@ -870,9 +896,9 @@ sub _poco_weeble_remove_request {
   if (defined $request) {
     DEBUG and warn "I/O: removed request $request_id";
     $request->remove_timeout();
+    delete $heap->{ext_request_to_int_id}{$request->[REQ_HTTP_REQUEST]};
     if (my $wheel = $request->wheel) {
       delete $heap->{wheel_to_request}->{$wheel->ID};
-      delete $heap->{request_to_id}{$request->[REQ_REQUEST]};
     }
   }
 }
@@ -882,7 +908,7 @@ sub _poco_weeble_remove_request {
 
 sub _poco_weeble_cancel {
   my ($kernel, $heap, $request) = @_[KERNEL, HEAP, ARG0];
-  my $request_id = $heap->{request_to_id}{$request};
+  my $request_id = $heap->{ext_request_to_int_id}{$request};
   return unless defined $request_id;
   _internal_cancel(
     $heap, $request_id, 408, "Request timed out (request canceled)"
@@ -895,14 +921,14 @@ sub _internal_cancel {
   my $request = delete $heap->{request}{$request_id};
   return unless defined $request;
 
-  DEBUG and warn "SHT: Shutdown is canceling request $request_id";
+  DEBUG and warn "CXL: canceling request $request_id";
   $request->remove_timeout();
+  delete $heap->{ext_request_to_int_id}{$request->[REQ_HTTP_REQUEST]};
 
   if (my $wheel = $request->wheel) {
     my $wheel_id = $wheel->ID;
-    DEBUG and warn "SHT: Request $request_id canceling wheel $wheel_id";
+    DEBUG and warn "CXL: Request $request_id canceling wheel $wheel_id";
     delete $heap->{wheel_to_request}{$wheel_id};
-    delete $heap->{request_to_id}{$request->[REQ_REQUEST]};
     $wheel = undef;
   }
 
@@ -917,7 +943,7 @@ sub _internal_cancel {
   }
 
   unless ($request->[REQ_STATE] & RS_POSTED) {
-    $request->error(408, "Request timed out (component shut down)");
+    $request->error($code, $message);
   }
 }
 
@@ -936,7 +962,7 @@ sub _poco_weeble_shutdown {
 
   # Shut down the connection manager subcomponent.
   if (defined $heap->{cm}) {
-    DEBUG and warn "SHT: Client::HTTP shutting down Client::Keepalive";
+    DEBUG and warn "CXL: Client::HTTP shutting down Client::Keepalive";
     $heap->{cm}->shutdown();
     delete $heap->{cm};
   }
@@ -967,9 +993,10 @@ POE::Component::Client::HTTP - a HTTP user-agent component
     Timeout   => 60,                    # defaults to 180 seconds
     MaxSize   => 16384,                 # defaults to entire response
     Streaming => 4096,                  # defaults to 0 (off)
-    FollowRedirects => 2                # defaults to 0 (off)
+    FollowRedirects => 2,               # defaults to 0 (off)
     Proxy     => "http://localhost:80", # defaults to HTTP_PROXY env. variable
     NoProxy   => [ "localhost", "127.0.0.1" ], # defs to NO_PROXY env. variable
+    BindAddr  => "12.34.56.78",         # defaults to INADDR_ANY
   );
 
   $kernel->post(
@@ -1019,9 +1046,9 @@ POE::Component::Client::HTTP is an HTTP user-agent for POE.  It lets
 other sessions run while HTTP transactions are being processed, and it
 lets several HTTP transactions be processed in parallel.
 
-If POE::Component::Client::DNS is also installed, Client::HTTP will
-use it to resolve hosts without blocking.  Otherwise it will use
-gethostbyname(), which may have performance problems.
+It supports keep-alive through POE::Component::Client::Keepalive,
+which in turn uses POE::Component::Client::DNS for asynchronous name
+resolution.
 
 HTTP client components are not proper objects.  Instead of being
 created, as most objects are, they are "spawned" as separate sessions.
@@ -1111,6 +1138,16 @@ NO_PROXY environment variable.
   NoProxy => [ "localhost", "127.0.0.1" ],
   NoProxy => "localhost,127.0.0.1",
 
+=item BindAddr => $local_ip
+
+Specify C<BindAddr> to bind all client sockets to a particular local
+address.  The value of BindAddr will be passed through
+POE::Component::Client::Keepalive to POE::Wheel::SocketFactory (as
+C<bind_address>).  See that module's documentation for implementation
+details.
+
+  BindAddr => "12.34.56.78"
+
 =item Protocol => $http_protocol_string
 
 C<Protocol> advertises the protocol that the client wishes to see.
@@ -1177,8 +1214,48 @@ for details here.
 
 =item Timeout => $query_timeout
 
-C<Timeout> specifies the amount of time a HTTP request will wait for
-an answer.  This defaults to 180 seconds (three minutes).
+C<Timeout> sets how long POE::Component::Client::HTTP has to process
+an application's request, in seconds.  C<Timeout> defaults to 180
+(three minutes) if not specified.
+
+It's important to note that the timeout begins when the component
+receives an application's request, not when it attempts to connect to
+the web server.
+
+Timeouts may result from sending the component too many requests at
+once.  Each request would need to be received and tracked in order.
+Consider this:
+
+  $_[KERNEL]->post(component => request => ...) for (1..15_000);
+
+15,000 requests are queued together in one enormous bolus.  The
+component would receive and initialize them in order.  The first
+socket activity wouldn't arrive until the 15,000th request was set up.
+If that took longer than C<Timeout>, then the requests that have
+waited too long would fail.
+
+C<ConnectionManager>'s own timeout and concurrency limits also affect
+how many requests may be processed at once.  For example, most of the
+15,000 requests would wait in the connection manager's pool until
+sockets become available.  Meanwhile, the C<Timeout> would be counting
+down.
+
+Applications may elect to control concurrency outside the component's
+C<Timeout>.  They may do so in a few ways.
+
+The easiest way is to limit the initial number of requests to
+something more manageable.  As responses arrive, the application
+should handle them and start new requests.  This limits concurrency to
+the initial request count.
+
+An application may also outsource job throttling to another module,
+such as POE::Component::JobQueue.
+
+In any case, C<Timeout> and C<ConnectionManager> may be tuned to
+maximize timeouts and concurrency limits.  This may help in some
+cases.  Developers should be aware that doing so will increase memory
+usage.  POE::Component::Client::HTTP and KeepAlive track requests in
+memory, while applications are free to keep pending requests on disk.
 
 =back
 
@@ -1300,11 +1377,11 @@ each time it is called, and an empty string when done.  Don't forget
 to set the Content-Length header correctly.  Example:
 
   my $request = HTTP::Request->new( PUT => 'http://...' );
-  
+
   my $file = '/path/to/large_file';
-  
+
   open my $fh, '<', $file;
-  
+
   my $upload_cb = sub {
     if ( sysread $fh, my $buf, 4096 ) {
       return $buf;
@@ -1314,13 +1391,44 @@ to set the Content-Length header correctly.  Example:
       return '';
     }
   };
-  
+
   $request->content_length( -s $file );
-  
+
   $request->content( $upload_cb );
-  
+
   $kernel->post( ua => request, 'response', $request );
 
+=head1 CONTENT ENCODING AND COMPRESSION
+
+Transparent content decoding has been disabled as of version 0.84.
+This also removes support for transparent gzip requesting and
+decompression.
+
+To re-enable gzip compression, specify the gzip Content-Encoding and
+use HTTP::Response's decoded_content() method rather than content():
+
+  my $request = HTTP::Request->new(
+    GET => "http://www.yahoo.com/", [
+      'Accept-Encoding' => 'gzip'
+    ]
+  );
+
+  # ... time passes ...
+
+  my $content = $response->decoded_content();
+
+The change in POE::Component::Client::HTTP behavior was prompted by
+changes in HTTP::Response that surfaced a bug in the component's
+transparent gzip handling.
+
+Allowing the application to specify and handle content encodings seems
+to be the most reliable and flexible resolution.
+
+For more information about the problem and discussions regarding the
+solution, see:
+L<http://www.perlmonks.org/?node_id=683833> and
+L<http://rt.cpan.org/Ticket/Display.html?id=35538>
+
 =head1 ENVIRONMENT
 
 POE::Component::Client::HTTP uses two standard environment variables:
@@ -1353,7 +1461,7 @@ There is no object oriented interface.  See
 L<POE::Component::Client::Keepalive> and
 L<POE::Component::Client::DNS> for examples of a decent OO interface.
 
-=head1 AUTHOR, COPYRIGHT, & LICENSE 
+=head1 AUTHOR, COPYRIGHT, & LICENSE
 
 POE::Component::Client::HTTP is
 
@@ -1399,3 +1507,4 @@ For questions, try the L<POE> mailing list (poe@perl.org)
 =cut
 
 # }}} POD
+# rocco // vim: ts=2 sw=2 expandtab
